from vivarium.library.path import open_relative

def read_sequence(path):
    with open_relative(path, 'r') as fasta:
        header = False
        sequence = ''
        for line in fasta:
            if not header:
                header = True
            else:
                sequence += line.strip()
    return sequence



""" Vivarium module init
Register processes, updaters, dividers, serializers upon import
"""

# import registries
from vivarium.core.registry import (
    process_registry,
    updater_registry,
    divider_registry,
    serializer_registry,
)

# import processes
from vivarium.processes.meta_division import MetaDivision
from vivarium.processes.tree_mass import TreeMass
from vivarium.processes.derive_concentrations import DeriveConcentrations
from vivarium.processes.derive_counts import DeriveCounts
from vivarium.processes.timeline import TimelineProcess
from vivarium.processes.nonspatial_environment import NonSpatialEnvironment

# import updaters, dividers, serializers
from vivarium.core.registry import (
    update_accumulate, update_set, update_merge, update_nonnegative_accumulate,
    divide_set, divide_split, divide_split_dict, divide_zero, assert_no_divide,
    NumpySerializer, NumpyScalarSerializer, UnitsSerializer, ProcessSerializer,
    GeneratorSerializer, FunctionSerializer
)


# register processes
process_registry.register(MetaDivision.name, MetaDivision)
process_registry.register(TreeMass.name, TreeMass)
process_registry.register(DeriveConcentrations.name, DeriveConcentrations)
process_registry.register(DeriveCounts.name, DeriveCounts)
process_registry.register(TimelineProcess.name, TimelineProcess)
process_registry.register(NonSpatialEnvironment.name, NonSpatialEnvironment)

# register updaters
updater_registry.register('accumulate', update_accumulate)
updater_registry.register('set', update_set)
updater_registry.register('merge', update_merge)
updater_registry.register('nonnegative_accumulate', update_nonnegative_accumulate)

# register dividers
divider_registry.register('set', divide_set)
divider_registry.register('split', divide_split)
divider_registry.register('split_dict', divide_split_dict)
divider_registry.register('zero', divide_zero)
divider_registry.register('no_divide', assert_no_divide)

# register serializers
serializer_registry.register('numpy', NumpySerializer())
serializer_registry.register('numpy_scalar', NumpyScalarSerializer())
serializer_registry.register('units', UnitsSerializer())
serializer_registry.register('process', ProcessSerializer())
serializer_registry.register('compartment', GeneratorSerializer())
serializer_registry.register('function', FunctionSerializer())



"""
================
Injector Process
================

Injectors model the appearance of substances within a cell or its
environment. For example, we could inject antibiotics into a cell. More
commonly, though, injector :term:`processes` let us build toy examples.
As such, we use injectors in the documentation to keep our examples
simple.
"""


from __future__ import absolute_import, division, print_function

import os

from vivarium.core.composition import (
    simulate_process,
    TEST_OUT_DIR,
    save_timeseries,
)
from vivarium.plots.simulation_output import plot_simulation_output
from vivarium.core.process import Process

NAME = 'injector'


class Injector(Process):

    name = NAME

    def __init__(self, initial_parameters=None):
        """Models the direct injection of substrates into a cell

        :term:`Ports`:

        * **internal**: The :term:`store` into which the substrates will
          be injected.

        .. note:: Each of these processes only supports injecting into a
            single store. To inject into multiple stores, create a
            separate instance of this injector process for each store.

        Arguments:
            initial_parameters (dict): An optional configuration
                dictionary that may include the key
                ``substrate_rate_map`` whose value must be a dictionary
                mapping substrate variable names to the rate (as a
                :py:class:`float`) at which they should be injected.
                Rates are interpreted as being in units of
                :math:`\\frac{substrateVariableUnit}{timestepUnit}`, so
                for a variable with units of molarity and a timestep in
                seconds, rates should be in :math:`\\frac{M}{s}`.
        """
        if initial_parameters is None:
            initial_parameters = {}

        self.substrate_rate_map = initial_parameters['substrate_rate_map']

        super(Injector, self).__init__(initial_parameters)

    def ports_schema(self):
        return {
            'internal': {
                substrate: {
                    '_default': 0,
                    '_emit': True,
                }
                for substrate in self.substrate_rate_map
            },
        }

    def next_update(self, timestep, states):
        return {
            'internal': {
                substrate: timestep * rate
                for substrate, rate in self.substrate_rate_map.items()
            }
        }


def run_injector():
    parameters = {
        'substrate_rate_map': {'toy': 1.0},
        }
    injector = Injector(parameters)
    settings = {
        'total_time': 10,
    }
    timeseries = simulate_process(injector, settings)
    return timeseries


def test_injector():
    timeseries = run_injector()
    # Expect [0, 1, ..., 10] because 0 at start
    expected = [i for i in range(11)]
    assert expected == timeseries['internal']['toy']


def main():
    out_dir = os.path.join(TEST_OUT_DIR, NAME)
    if not os.path.exists(out_dir):
        os.makedirs(out_dir)
    timeseries = run_injector()
    plot_settings = {}
    plot_simulation_output(timeseries, plot_settings, out_dir)
    save_timeseries(timeseries, out_dir)


if __name__ == '__main__':
    main()
